#include <windows.h>
#include "brush.h"
#include "tll.h"
#include "texture.h"


extern int numBrushes;
extern POLYGON* brushPolygons;
extern BRUSH* BrushSet;
extern BRUSH* Brush;
extern VERTEX* vertex1;
extern VERTEX* vertex2;
extern VERTEX* vertex3;
extern TEXTURE* texture;
extern bool Texturing;
extern bool CSGflag;
extern bool CSGcullface;

BRUSH::BRUSH()
{
    NextBrush = NULL;
    numPolygons    = 0;
    id = 0;
}

BRUSH::~BRUSH()
{
}

bool BRUSH::IsLast()
{
    if (NextBrush == NULL)
        return true;
    else
        return false;
}

BRUSH* BRUSH::GetNext()
{
    if (!IsLast())
        return NextBrush;
    else
        return NULL;
}

void BRUSH::SetNext(BRUSH* Brush)
{
    if (IsLast())
    {
        NextBrush = Brush;
        return;
    }

    BRUSH* TempBrush = Brush;

    while (!TempBrush->IsLast())
        TempBrush = TempBrush->GetNext();

    TempBrush->SetNext(NextBrush);
    NextBrush = Brush;
}

BRUSH* BRUSH::CopyList()
{
    BRUSH* TempBrush = new BRUSH;

    TempBrush->id = id;

    TempBrush->numPolygons = numPolygons;

    for (int loop = 0; loop < numPolygons; loop++)
        TempBrush->Polygon[loop] = Polygon[loop];

    if (!IsLast())
        TempBrush->AddBrush(NextBrush->CopyList());

    return TempBrush;
}

void BRUSH::AddBrush(BRUSH* Brush)
{
    if (Brush != NULL)
    {
        if (IsLast())
        {
            NextBrush = Brush;
            return;
        }

        BRUSH* TempBrush = NextBrush;

        while (!TempBrush->IsLast())
            TempBrush = TempBrush->GetNext();

        TempBrush->NextBrush = Brush;
    }
}

int BRUSH::Compare(const BRUSH& Brush)
{
    if (linkPosition < Brush.linkPosition)
            return smaller;
      if (linkPosition > Brush.linkPosition)
            return bigger;
      else
            return same;
}

void DeleteList(BRUSH* Brush)
{
    if (Brush->IsLast())
    {
        delete Brush;
        return;
    }
    
    DeleteList(Brush->GetNext());
    delete Brush;
}

void InvertBrushPolygons(BRUSH* Brush)
{
    int numverts;
    VERTEX tempvertices[maxPolygonVerts];
    for (int loop = 0; loop < Brush->numPolygons; loop++)
    {
        numverts = Brush->Polygon[loop].numVertices;
        for (int innerloop = 0; innerloop < numverts; innerloop++)
            tempvertices[innerloop] = Brush->Polygon[loop].Vertex[innerloop];
        for (int innerloop = 0; innerloop < numverts; innerloop++)
            Brush->Polygon[loop].Vertex[innerloop] = tempvertices[(numverts - 1) - innerloop];
        Brush->Polygon[loop].SetNormal();
    }
    return;
}

// Brush vertex and polygon initialization
void Set1stVertices()
{
// Set cubes vertices

    vertex1[0].coords.x = 20;
    vertex1[0].coords.y = 5;
    vertex1[0].coords.z = 30;

    vertex1[1].coords.x = -20;
    vertex1[1].coords.y = 5;
    vertex1[1].coords.z = 30;

    vertex1[2].coords.x = -20;
    vertex1[2].coords.y = 5;
    vertex1[2].coords.z = 10;

    vertex1[3].coords.x = 20;
    vertex1[3].coords.y = 5;
    vertex1[3].coords.z = 10;

    vertex1[4].coords.x = 20;
    vertex1[4].coords.y = -5;
    vertex1[4].coords.z = 30;

    vertex1[5].coords.x = -20;
    vertex1[5].coords.y = -5;
    vertex1[5].coords.z = 30;

    vertex1[6].coords.x = -20;
    vertex1[6].coords.y = -5;
    vertex1[6].coords.z = 10;

    vertex1[7].coords.x = 20;
    vertex1[7].coords.y = -5;
    vertex1[7].coords.z = 10;
}

void Set2ndVertices()
{
// Set cubes vertices
    vertex2[0].coords.x = -20;
    vertex2[0].coords.y = -10;
    vertex2[0].coords.z = 20;

    vertex2[1].coords.x = 20;
    vertex2[1].coords.y = -10;
    vertex2[1].coords.z = 20;

    vertex2[2].coords.x = 20;
    vertex2[2].coords.y = 10;
    vertex2[2].coords.z = 20;

    vertex2[3].coords.x = -20;
    vertex2[3].coords.y = 10;
    vertex2[3].coords.z = 20;

    vertex2[4].coords.x = -20;
    vertex2[4].coords.y = -10;
    vertex2[4].coords.z = -20;

    vertex2[5].coords.x = 20;
    vertex2[5].coords.y = -10;
    vertex2[5].coords.z = -20;

    vertex2[6].coords.x = 20;
    vertex2[6].coords.y = 10;
    vertex2[6].coords.z = -20;

    vertex2[7].coords.x = -20;
    vertex2[7].coords.y = 10;
    vertex2[7].coords.z = -20;
}

void Set3rdVertices()
{
// Set cubes vertices
    vertex3[0].coords.x = -30;
    vertex3[0].coords.y = -5;
    vertex3[0].coords.z = 10;

    vertex3[1].coords.x = 5;
    vertex3[1].coords.y = -5;
    vertex3[1].coords.z = 10;

    vertex3[2].coords.x = 5;
    vertex3[2].coords.y = 20;
    vertex3[2].coords.z = 10;

    vertex3[3].coords.x = -30;
    vertex3[3].coords.y = 20;
    vertex3[3].coords.z = 10;

    vertex3[4].coords.x = -30;
    vertex3[4].coords.y = -5;
    vertex3[4].coords.z = -10;

    vertex3[5].coords.x = 5;
    vertex3[5].coords.y = -5;
    vertex3[5].coords.z = -10;

    vertex3[6].coords.x = 5;
    vertex3[6].coords.y = 20;
    vertex3[6].coords.z = -10;

    vertex3[7].coords.x = -30;
    vertex3[7].coords.y = 20;
    vertex3[7].coords.z = -10;
}

void SetBrushPolygons()
{
// Set brush 1 polygons

    Brush[0].Polygon[0].Vertex[0] = vertex1[1];
    Brush[0].Polygon[0].Vertex[1] = vertex1[0];
    Brush[0].Polygon[0].Vertex[2] = vertex1[3];
    Brush[0].Polygon[0].Vertex[3] = vertex1[2];
    Brush[0].Polygon[0].Vertex[0].u = 0.0;
    Brush[0].Polygon[0].Vertex[0].v = 0.0;
    Brush[0].Polygon[0].Vertex[1].u = 1.0;
    Brush[0].Polygon[0].Vertex[1].v = 0.0;
    Brush[0].Polygon[0].Vertex[2].u = 1.0;
    Brush[0].Polygon[0].Vertex[2].v = 1.0;
    Brush[0].Polygon[0].Vertex[3].u = 0.0;
    Brush[0].Polygon[0].Vertex[3].v = 1.0;
    Brush[0].Polygon[0].Vertex[0].polyNumber = 0;
    Brush[0].Polygon[0].Vertex[1].polyNumber = 0;
    Brush[0].Polygon[0].Vertex[2].polyNumber = 0;
    Brush[0].Polygon[0].Vertex[3].polyNumber = 0;
    Brush[0].Polygon[0].Texture = texture[10].TexID;

    Brush[0].Polygon[1].Vertex[0] = vertex1[5];
    Brush[0].Polygon[1].Vertex[1] = vertex1[4];
    Brush[0].Polygon[1].Vertex[2] = vertex1[0];
    Brush[0].Polygon[1].Vertex[3] = vertex1[1];
    Brush[0].Polygon[1].Vertex[0].u = 0.0;
    Brush[0].Polygon[1].Vertex[0].v = 0.0;
    Brush[0].Polygon[1].Vertex[1].u = 1.0;
    Brush[0].Polygon[1].Vertex[1].v = 0.0;
    Brush[0].Polygon[1].Vertex[2].u = 1.0;
    Brush[0].Polygon[1].Vertex[2].v = 1.0;
    Brush[0].Polygon[1].Vertex[3].u = 0.0;
    Brush[0].Polygon[1].Vertex[3].v = 1.0;
    Brush[0].Polygon[1].Vertex[0].polyNumber = 3;
    Brush[0].Polygon[1].Vertex[1].polyNumber = 3;
    Brush[0].Polygon[1].Vertex[2].polyNumber = 3;
    Brush[0].Polygon[1].Vertex[3].polyNumber = 3;
    Brush[0].Polygon[1].Texture = texture[10].TexID;

    Brush[0].Polygon[2].Vertex[0] = vertex1[6];
    Brush[0].Polygon[2].Vertex[1] = vertex1[5];
    Brush[0].Polygon[2].Vertex[2] = vertex1[1];
    Brush[0].Polygon[2].Vertex[3] = vertex1[2];
    Brush[0].Polygon[2].Vertex[0].u = 0.0;
    Brush[0].Polygon[2].Vertex[0].v = 0.0;
    Brush[0].Polygon[2].Vertex[1].u = 1.0;
    Brush[0].Polygon[2].Vertex[1].v = 0.0;
    Brush[0].Polygon[2].Vertex[2].u = 1.0;
    Brush[0].Polygon[2].Vertex[2].v = 1.0;
    Brush[0].Polygon[2].Vertex[3].u = 0.0;
    Brush[0].Polygon[2].Vertex[3].v = 1.0;
    Brush[0].Polygon[2].Vertex[0].polyNumber = 4;
    Brush[0].Polygon[2].Vertex[1].polyNumber = 4;
    Brush[0].Polygon[2].Vertex[2].polyNumber = 4;
    Brush[0].Polygon[2].Vertex[3].polyNumber = 4;
    Brush[0].Polygon[2].Texture = texture[10].TexID;

    Brush[0].Polygon[3].Vertex[0] = vertex1[7];
    Brush[0].Polygon[3].Vertex[1] = vertex1[6];
    Brush[0].Polygon[3].Vertex[2] = vertex1[2];
    Brush[0].Polygon[3].Vertex[3] = vertex1[3];
    Brush[0].Polygon[3].Vertex[0].u = 0.0;
    Brush[0].Polygon[3].Vertex[0].v = 0.0;
    Brush[0].Polygon[3].Vertex[1].u = 1.0;
    Brush[0].Polygon[3].Vertex[1].v = 0.0;
    Brush[0].Polygon[3].Vertex[2].u = 1.0;
    Brush[0].Polygon[3].Vertex[2].v = 1.0;
    Brush[0].Polygon[3].Vertex[3].u = 0.0;
    Brush[0].Polygon[3].Vertex[3].v = 1.0;
    Brush[0].Polygon[3].Vertex[0].polyNumber = 9;
    Brush[0].Polygon[3].Vertex[1].polyNumber = 9;
    Brush[0].Polygon[3].Vertex[2].polyNumber = 9;
    Brush[0].Polygon[3].Vertex[3].polyNumber = 9;
    Brush[0].Polygon[3].Texture = texture[10].TexID;

    Brush[0].Polygon[4].Vertex[0] = vertex1[4];
    Brush[0].Polygon[4].Vertex[1] = vertex1[7];
    Brush[0].Polygon[4].Vertex[2] = vertex1[3];
    Brush[0].Polygon[4].Vertex[3] = vertex1[0];
    Brush[0].Polygon[4].Vertex[0].u = 0.0;
    Brush[0].Polygon[4].Vertex[0].v = 0.0;
    Brush[0].Polygon[4].Vertex[1].u = 1.0;
    Brush[0].Polygon[4].Vertex[1].v = 0.0;
    Brush[0].Polygon[4].Vertex[2].u = 1.0;
    Brush[0].Polygon[4].Vertex[2].v = 1.0;
    Brush[0].Polygon[4].Vertex[3].u = 0.0;
    Brush[0].Polygon[4].Vertex[3].v = 1.0;
    Brush[0].Polygon[4].Vertex[0].polyNumber = 10;
    Brush[0].Polygon[4].Vertex[1].polyNumber = 10;
    Brush[0].Polygon[4].Vertex[2].polyNumber = 10;
    Brush[0].Polygon[4].Vertex[3].polyNumber = 10;
    Brush[0].Polygon[4].Texture = texture[10].TexID;

    Brush[0].Polygon[5].Vertex[0] = vertex1[5];
    Brush[0].Polygon[5].Vertex[1] = vertex1[6];
    Brush[0].Polygon[5].Vertex[2] = vertex1[7];
    Brush[0].Polygon[5].Vertex[3] = vertex1[4];
    Brush[0].Polygon[5].Vertex[0].u = 0.0;
    Brush[0].Polygon[5].Vertex[0].v = 0.0;
    Brush[0].Polygon[5].Vertex[1].u = 1.0;
    Brush[0].Polygon[5].Vertex[1].v = 0.0;
    Brush[0].Polygon[5].Vertex[2].u = 1.0;
    Brush[0].Polygon[5].Vertex[2].v = 1.0;
    Brush[0].Polygon[5].Vertex[3].u = 0.0;
    Brush[0].Polygon[5].Vertex[3].v = 1.0;
    Brush[0].Polygon[5].Vertex[0].polyNumber = 11;
    Brush[0].Polygon[5].Vertex[1].polyNumber = 11;
    Brush[0].Polygon[5].Vertex[2].polyNumber = 11;
    Brush[0].Polygon[5].Vertex[3].polyNumber = 11;
    Brush[0].Polygon[5].Texture = texture[10].TexID;

    for (int loop = 0; loop < Brush[0].numPolygons; loop++)
    {
        Brush[0].Polygon[loop].SetNormal();
    }

// Set brush 2 polygons

//Front
    Brush[1].Polygon[0].Vertex[0] = vertex2[0];
    Brush[1].Polygon[0].Vertex[1] = vertex2[1];
    Brush[1].Polygon[0].Vertex[2] = vertex2[2];
    Brush[1].Polygon[0].Vertex[3] = vertex2[3];
    Brush[1].Polygon[0].Vertex[0].u = 0.0;
    Brush[1].Polygon[0].Vertex[0].v = 0.0;
    Brush[1].Polygon[0].Vertex[1].u = 1.0;
    Brush[1].Polygon[0].Vertex[1].v = 0.0;
    Brush[1].Polygon[0].Vertex[2].u = 1.0;
    Brush[1].Polygon[0].Vertex[2].v = 1.0;
    Brush[1].Polygon[0].Vertex[3].u = 0.0;
    Brush[1].Polygon[0].Vertex[3].v = 1.0;
    Brush[1].Polygon[0].Vertex[0].polyNumber = 14;
    Brush[1].Polygon[0].Vertex[1].polyNumber = 14;
    Brush[1].Polygon[0].Vertex[2].polyNumber = 14;
    Brush[1].Polygon[0].Vertex[3].polyNumber = 14;
    Brush[1].Polygon[0].Texture = texture[10].TexID;

//Back
    Brush[1].Polygon[1].Vertex[0] = vertex2[5];
    Brush[1].Polygon[1].Vertex[1] = vertex2[4];
    Brush[1].Polygon[1].Vertex[2] = vertex2[7];
    Brush[1].Polygon[1].Vertex[3] = vertex2[6];
    Brush[1].Polygon[1].Vertex[0].u = 0.0;
    Brush[1].Polygon[1].Vertex[0].v = 0.0;
    Brush[1].Polygon[1].Vertex[1].u = 1.0;
    Brush[1].Polygon[1].Vertex[1].v = 0.0;
    Brush[1].Polygon[1].Vertex[2].u = 1.0;
    Brush[1].Polygon[1].Vertex[2].v = 1.0;
    Brush[1].Polygon[1].Vertex[3].u = 0.0;
    Brush[1].Polygon[1].Vertex[3].v = 1.0;
    Brush[1].Polygon[1].Vertex[0].polyNumber = 15;
    Brush[1].Polygon[1].Vertex[1].polyNumber = 15;
    Brush[1].Polygon[1].Vertex[2].polyNumber = 15;
    Brush[1].Polygon[1].Vertex[3].polyNumber = 15;
    Brush[1].Polygon[1].Texture = texture[10].TexID;

//Left
    Brush[1].Polygon[2].Vertex[0] = vertex2[4];
    Brush[1].Polygon[2].Vertex[1] = vertex2[0];
    Brush[1].Polygon[2].Vertex[2] = vertex2[3];
    Brush[1].Polygon[2].Vertex[3] = vertex2[7];
    Brush[1].Polygon[2].Vertex[0].u = 0.0;
    Brush[1].Polygon[2].Vertex[0].v = 0.0;
    Brush[1].Polygon[2].Vertex[1].u = 1.0;
    Brush[1].Polygon[2].Vertex[1].v = 0.0;
    Brush[1].Polygon[2].Vertex[2].u = 1.0;
    Brush[1].Polygon[2].Vertex[2].v = 1.0;
    Brush[1].Polygon[2].Vertex[3].u = 0.0;
    Brush[1].Polygon[2].Vertex[3].v = 1.0;
    Brush[1].Polygon[2].Vertex[0].polyNumber = 16;
    Brush[1].Polygon[2].Vertex[1].polyNumber = 16;
    Brush[1].Polygon[2].Vertex[2].polyNumber = 16;
    Brush[1].Polygon[2].Vertex[3].polyNumber = 16;
    Brush[1].Polygon[2].Texture = texture[10].TexID;

//Right
    Brush[1].Polygon[3].Vertex[0] = vertex2[1];
    Brush[1].Polygon[3].Vertex[1] = vertex2[5];
    Brush[1].Polygon[3].Vertex[2] = vertex2[6];
    Brush[1].Polygon[3].Vertex[3] = vertex2[2];
    Brush[1].Polygon[3].Vertex[0].u = 0.0;
    Brush[1].Polygon[3].Vertex[0].v = 0.0;
    Brush[1].Polygon[3].Vertex[1].u = 1.0;
    Brush[1].Polygon[3].Vertex[1].v = 0.0;
    Brush[1].Polygon[3].Vertex[2].u = 1.0;
    Brush[1].Polygon[3].Vertex[2].v = 1.0;
    Brush[1].Polygon[3].Vertex[3].u = 0.0;
    Brush[1].Polygon[3].Vertex[3].v = 1.0;
    Brush[1].Polygon[3].Vertex[0].polyNumber = 17;
    Brush[1].Polygon[3].Vertex[1].polyNumber = 17;
    Brush[1].Polygon[3].Vertex[2].polyNumber = 17;
    Brush[1].Polygon[3].Vertex[3].polyNumber = 17;
    Brush[1].Polygon[3].Texture = texture[10].TexID;

//Top
    Brush[1].Polygon[4].Vertex[0] = vertex2[3];
    Brush[1].Polygon[4].Vertex[1] = vertex2[2];
    Brush[1].Polygon[4].Vertex[2] = vertex2[6];
    Brush[1].Polygon[4].Vertex[3] = vertex2[7];
    Brush[1].Polygon[4].Vertex[0].u = 0.0;
    Brush[1].Polygon[4].Vertex[0].v = 0.0;
    Brush[1].Polygon[4].Vertex[1].u = 1.0;
    Brush[1].Polygon[4].Vertex[1].v = 0.0;
    Brush[1].Polygon[4].Vertex[2].u = 1.0;
    Brush[1].Polygon[4].Vertex[2].v = 1.0;
    Brush[1].Polygon[4].Vertex[3].u = 0.0;
    Brush[1].Polygon[4].Vertex[3].v = 1.0;
    Brush[1].Polygon[4].Vertex[0].polyNumber = 18;
    Brush[1].Polygon[4].Vertex[1].polyNumber = 18;
    Brush[1].Polygon[4].Vertex[2].polyNumber = 18;
    Brush[1].Polygon[4].Vertex[3].polyNumber = 18;
    Brush[1].Polygon[4].Texture = texture[10].TexID;

//Bottom
    Brush[1].Polygon[5].Vertex[0] = vertex2[5];
    Brush[1].Polygon[5].Vertex[1] = vertex2[1];
    Brush[1].Polygon[5].Vertex[2] = vertex2[0];
    Brush[1].Polygon[5].Vertex[3] = vertex2[4];
    Brush[1].Polygon[5].Vertex[0].u = 0.0;
    Brush[1].Polygon[5].Vertex[0].v = 0.0;
    Brush[1].Polygon[5].Vertex[1].u = 1.0;
    Brush[1].Polygon[5].Vertex[1].v = 0.0;
    Brush[1].Polygon[5].Vertex[2].u = 1.0;
    Brush[1].Polygon[5].Vertex[2].v = 1.0;
    Brush[1].Polygon[5].Vertex[3].u = 0.0;
    Brush[1].Polygon[5].Vertex[3].v = 1.0;
    Brush[1].Polygon[5].Vertex[0].polyNumber = 19;
    Brush[1].Polygon[5].Vertex[1].polyNumber = 19;
    Brush[1].Polygon[5].Vertex[2].polyNumber = 19;
    Brush[1].Polygon[5].Vertex[3].polyNumber = 19;
    Brush[1].Polygon[5].Texture = texture[10].TexID;

    for (int loop = 0; loop < Brush[1].numPolygons; loop++)
    {
        Brush[1].Polygon[loop].SetNormal();
    }

// Set brush 3 polygons

//Front
    Brush[2].Polygon[0].Vertex[0] = vertex3[0];
    Brush[2].Polygon[0].Vertex[1] = vertex3[1];
    Brush[2].Polygon[0].Vertex[2] = vertex3[2];
    Brush[2].Polygon[0].Vertex[3] = vertex3[3];
    Brush[2].Polygon[0].Vertex[0].u = 0.0;
    Brush[2].Polygon[0].Vertex[0].v = 0.0;
    Brush[2].Polygon[0].Vertex[1].u = 1.0;
    Brush[2].Polygon[0].Vertex[1].v = 0.0;
    Brush[2].Polygon[0].Vertex[2].u = 1.0;
    Brush[2].Polygon[0].Vertex[2].v = 1.0;
    Brush[2].Polygon[0].Vertex[3].u = 0.0;
    Brush[2].Polygon[0].Vertex[3].v = 1.0;
    Brush[2].Polygon[0].Vertex[0].polyNumber = 14;
    Brush[2].Polygon[0].Vertex[1].polyNumber = 14;
    Brush[2].Polygon[0].Vertex[2].polyNumber = 14;
    Brush[2].Polygon[0].Vertex[3].polyNumber = 14;
    Brush[2].Polygon[0].Texture = texture[10].TexID;

//Back
    Brush[2].Polygon[1].Vertex[0] = vertex3[5];
    Brush[2].Polygon[1].Vertex[1] = vertex3[4];
    Brush[2].Polygon[1].Vertex[2] = vertex3[7];
    Brush[2].Polygon[1].Vertex[3] = vertex3[6];
    Brush[2].Polygon[1].Vertex[0].u = 0.0;
    Brush[2].Polygon[1].Vertex[0].v = 0.0;
    Brush[2].Polygon[1].Vertex[1].u = 1.0;
    Brush[2].Polygon[1].Vertex[1].v = 0.0;
    Brush[2].Polygon[1].Vertex[2].u = 1.0;
    Brush[2].Polygon[1].Vertex[2].v = 1.0;
    Brush[2].Polygon[1].Vertex[3].u = 0.0;
    Brush[2].Polygon[1].Vertex[3].v = 1.0;
    Brush[2].Polygon[1].Vertex[0].polyNumber = 15;
    Brush[2].Polygon[1].Vertex[1].polyNumber = 15;
    Brush[2].Polygon[1].Vertex[2].polyNumber = 15;
    Brush[2].Polygon[1].Vertex[3].polyNumber = 15;
    Brush[2].Polygon[1].Texture = texture[10].TexID;

//Left
    Brush[2].Polygon[2].Vertex[0] = vertex3[4];
    Brush[2].Polygon[2].Vertex[1] = vertex3[0];
    Brush[2].Polygon[2].Vertex[2] = vertex3[3];
    Brush[2].Polygon[2].Vertex[3] = vertex3[7];
    Brush[2].Polygon[2].Vertex[0].u = 0.0;
    Brush[2].Polygon[2].Vertex[0].v = 0.0;
    Brush[2].Polygon[2].Vertex[1].u = 1.0;
    Brush[2].Polygon[2].Vertex[1].v = 0.0;
    Brush[2].Polygon[2].Vertex[2].u = 1.0;
    Brush[2].Polygon[2].Vertex[2].v = 1.0;
    Brush[2].Polygon[2].Vertex[3].u = 0.0;
    Brush[2].Polygon[2].Vertex[3].v = 1.0;
    Brush[2].Polygon[2].Vertex[0].polyNumber = 16;
    Brush[2].Polygon[2].Vertex[1].polyNumber = 16;
    Brush[2].Polygon[2].Vertex[2].polyNumber = 16;
    Brush[2].Polygon[2].Vertex[3].polyNumber = 16;
    Brush[2].Polygon[2].Texture = texture[10].TexID;

//Right
    Brush[2].Polygon[3].Vertex[0] = vertex3[1];
    Brush[2].Polygon[3].Vertex[1] = vertex3[5];
    Brush[2].Polygon[3].Vertex[2] = vertex3[6];
    Brush[2].Polygon[3].Vertex[3] = vertex3[2];
    Brush[2].Polygon[3].Vertex[0].u = 0.0;
    Brush[2].Polygon[3].Vertex[0].v = 0.0;
    Brush[2].Polygon[3].Vertex[1].u = 1.0;
    Brush[2].Polygon[3].Vertex[1].v = 0.0;
    Brush[2].Polygon[3].Vertex[2].u = 1.0;
    Brush[2].Polygon[3].Vertex[2].v = 1.0;
    Brush[2].Polygon[3].Vertex[3].u = 0.0;
    Brush[2].Polygon[3].Vertex[3].v = 1.0;
    Brush[2].Polygon[3].Vertex[0].polyNumber = 17;
    Brush[2].Polygon[3].Vertex[1].polyNumber = 17;
    Brush[2].Polygon[3].Vertex[2].polyNumber = 17;
    Brush[2].Polygon[3].Vertex[3].polyNumber = 17;
    Brush[2].Polygon[3].Texture = texture[10].TexID;

//Top
    Brush[2].Polygon[4].Vertex[0] = vertex3[3];
    Brush[2].Polygon[4].Vertex[1] = vertex3[2];
    Brush[2].Polygon[4].Vertex[2] = vertex3[6];
    Brush[2].Polygon[4].Vertex[3] = vertex3[7];
    Brush[2].Polygon[4].Vertex[0].u = 0.0;
    Brush[2].Polygon[4].Vertex[0].v = 0.0;
    Brush[2].Polygon[4].Vertex[1].u = 1.0;
    Brush[2].Polygon[4].Vertex[1].v = 0.0;
    Brush[2].Polygon[4].Vertex[2].u = 1.0;
    Brush[2].Polygon[4].Vertex[2].v = 1.0;
    Brush[2].Polygon[4].Vertex[3].u = 0.0;
    Brush[2].Polygon[4].Vertex[3].v = 1.0;
    Brush[2].Polygon[4].Vertex[0].polyNumber = 18;
    Brush[2].Polygon[4].Vertex[1].polyNumber = 18;
    Brush[2].Polygon[4].Vertex[2].polyNumber = 18;
    Brush[2].Polygon[4].Vertex[3].polyNumber = 18;
    Brush[2].Polygon[4].Texture = texture[10].TexID;

//Bottom
    Brush[2].Polygon[5].Vertex[0] = vertex3[5];
    Brush[2].Polygon[5].Vertex[1] = vertex3[1];
    Brush[2].Polygon[5].Vertex[2] = vertex3[0];
    Brush[2].Polygon[5].Vertex[3] = vertex3[4];
    Brush[2].Polygon[5].Vertex[0].u = 0.0;
    Brush[2].Polygon[5].Vertex[0].v = 0.0;
    Brush[2].Polygon[5].Vertex[1].u = 1.0;
    Brush[2].Polygon[5].Vertex[1].v = 0.0;
    Brush[2].Polygon[5].Vertex[2].u = 1.0;
    Brush[2].Polygon[5].Vertex[2].v = 1.0;
    Brush[2].Polygon[5].Vertex[3].u = 0.0;
    Brush[2].Polygon[5].Vertex[3].v = 1.0;
    Brush[2].Polygon[5].Vertex[0].polyNumber = 19;
    Brush[2].Polygon[5].Vertex[1].polyNumber = 19;
    Brush[2].Polygon[5].Vertex[2].polyNumber = 19;
    Brush[2].Polygon[5].Vertex[3].polyNumber = 19;
    Brush[2].Polygon[5].Texture = texture[10].TexID;

    for (int loop = 0; loop < Brush[2].numPolygons; loop++)
    {
        Brush[2].Polygon[loop].SetNormal();
    }
}


void CreateBrushSet()
{
    for (int loop = 0; loop < numBrushes; loop++)
    {
        BRUSH* newBrush = new BRUSH;
        *newBrush = Brush[loop];
        if (BrushSet)
            BrushSet->AddBrush(newBrush);            
        else
            BrushSet = newBrush;
    }
}

void ResetBrushes()
{
    if (CSGflag)
        return;

    Brush[0].numPolygons = 6;
    Brush[1].numPolygons = 6;
    Brush[2].numPolygons = 6;
    Brush[0].id = 1;
    Brush[1].id = 2;
    Brush[2].id = 3;

    for (int loop = 0; loop < Brush[0].numPolygons; loop++)
    {
        Brush[0].Polygon[loop].id = loop;
        Brush[0].Polygon[loop].numVertices = 4;
    }

    for (int loop = 0; loop < Brush[1].numPolygons; loop++)
    {
        Brush[1].Polygon[loop].id = loop;
        Brush[1].Polygon[loop].numVertices = 4;
    }

    for (int loop = 0; loop < Brush[2].numPolygons; loop++)
    {
        Brush[2].Polygon[loop].id = loop;
        Brush[2].Polygon[loop].numVertices = 4;
    }
    Set1stVertices();
    Set2ndVertices();
    Set3rdVertices();
    SetBrushPolygons();

    if (BrushSet)
    {
        DeleteList(BrushSet);
        BrushSet = NULL;
    }
    CreateBrushSet();

    CSGflag = 1;
    return;
}

void RenderWireframeBrush(BRUSH* Brush)
{
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glColor3f(1.0, 1.0, 0.0); 

    for (int loop = 0; loop < Brush->numPolygons; loop++)
    {
        glBegin(GL_LINES);
            for (int innerloop = 0; innerloop < Brush->Polygon[loop].numVertices - 1; innerloop++)
            {
                glVertex3fv(&Brush->Polygon[loop].Vertex[innerloop].coords.x);
                glVertex3fv(&Brush->Polygon[loop].Vertex[innerloop + 1].coords.x);
            }
            glVertex3fv(&Brush->Polygon[loop].Vertex[Brush->Polygon[loop].numVertices - 1].coords.x);
            glVertex3fv(&Brush->Polygon[loop].Vertex[0].coords.x);
        glEnd();
    }
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_LIGHTING);
}

void RenderTexturedBrush(BRUSH* Brush)
{
    for (int loop = 0; loop < Brush->numPolygons; loop++)
    {
        glBindTexture(GL_TEXTURE_2D, Brush->Polygon[loop].Texture);
        glBegin(GL_POLYGON);
            glNormal3fv(&Brush->Polygon[loop].Vertex[0].normal.x);
            for (int innerloop = 0; innerloop < Brush->Polygon[loop].numVertices; innerloop++)
            {
                   glTexCoord2f(Brush->Polygon[loop].Vertex[innerloop].u, Brush->Polygon[loop].Vertex[innerloop].v);
                glVertex3fv(&Brush->Polygon[loop].Vertex[innerloop].coords.x);
            }
            glTexCoord2f(Brush->Polygon[loop].Vertex[0].u, Brush->Polygon[loop].Vertex[0].v);
            glVertex3fv(&Brush->Polygon[loop].Vertex[0].coords.x);
        glEnd();
    }
}

void DrawBrushes()
{
    glPushMatrix();

    if (CSGcullface)
        glDisable(GL_CULL_FACE);

    if (Texturing)
        RenderTexturedBrush(&Brush[0]);
    else
        RenderWireframeBrush(&Brush[0]);

    if (CSGflag)
    {
        if (Texturing)
        {
            RenderTexturedBrush(&Brush[1]);
            RenderTexturedBrush(&Brush[2]);
        }
        else
        {
            RenderWireframeBrush(&Brush[1]);
            RenderWireframeBrush(&Brush[2]);
        }
    }

    glEnable(GL_CULL_FACE);
    glPopMatrix();
}
