#include <windows.h>
#include "bullet.h"
#include "decal.h"
#include "vector.h"
#include "vertex.h"
#include "polygon.h"
#include "texture.h"
#include "listnode.h"
#include "collision.h"
#include "general.h"
#include "camera.h"
#include "bsp.h"
#include "tll.h"


extern VERTEX* vertex;
extern TEXTURE* texture;
extern int numBullets;
extern BULLET* bullet;
extern int numBulletsActive;
extern int numleaves;
extern LinkedList<BSP_node> LeafList;
extern LinkedList<BSP_node> PartitionList;
extern BSP_node* listnode;
extern CAMERA* camera;
extern int currentCamera;
extern int bulletspeed;
extern int bullettype;
extern BSP_node* root;
extern int maxDecals;
extern DECAL muzzleflashdecal;
extern DECAL impactflashdecal;

DECAL::DECAL()
{
}

DECAL::~DECAL()
{
}

int DECAL::Compare(const DECAL& Decal)
{
    if (linkPosition < Decal.linkPosition)
            return smaller;
      if (linkPosition > Decal.linkPosition)
            return bigger;
      else
            return same;
}

// the polygon (face) passed in must be a triangle
void CreateDecal(VECTOR pt, POLYGON* face, DECAL* decal)
{
    VECTOR axis[3] =
    {
        VECTOR(1.0f, 0.0f, 0.0f),
        VECTOR(0.0f, 1.0f, 0.0f),
        VECTOR(0.0f, 0.0f, 1.0f)
    };
    VECTOR poly_normal(fabs(face->Vertex[0].normal.x), fabs(face->Vertex[0].normal.y), fabs(face->Vertex[0].normal.z));
    poly_normal.Normalize();
    VECTOR up, right, temp;
    int major;

    if (poly_normal.x > poly_normal.y && poly_normal.x > poly_normal.z)
        major = 0;
    else if (poly_normal.y > poly_normal.x && poly_normal.y > poly_normal.z)
        major = 1;
    else
        major = 2;

    // build right vector by hand
    if (poly_normal.x == 1.0 || poly_normal.y == 1.0 || poly_normal.z == 1.0)
    {
        if ((major == 0 && face->Vertex[0].normal.x > 0.0) || major == 1)
            right.Set(0.0, 0.0, -1.0);
        else if (major == 0)
            right.Set(0.0, 0.0, 1.0);
        else
            right.Set(1.0 * face->Vertex[0].normal.z, 0.0, 0.0);
    }
    else
    {
        temp = face->Vertex[0].normal;
        axis[major].CrossVector(temp);
        right = axis[major];
    }

    temp.x = -face->Vertex[0].normal.x;
    temp.y = -face->Vertex[0].normal.y;
    temp.z = -face->Vertex[0].normal.z;
    temp.CrossVector(right);
    up = temp;
    up.Normalize();
    right.Normalize();

   // make the face
   decal->Vertex[0].coords.x = pt.x + ((-right.x - up.x) * decal->Size);
   decal->Vertex[0].coords.y = pt.y + ((-right.y - up.y) * decal->Size);
   decal->Vertex[0].coords.z = pt.z + ((-right.z - up.z) * decal->Size);
   decal->Vertex[1].coords.x = pt.x + ((right.x - up.x) * decal->Size);
   decal->Vertex[1].coords.y = pt.y + ((right.y - up.y) * decal->Size);
   decal->Vertex[1].coords.z = pt.z + ((right.z - up.z) * decal->Size);
   decal->Vertex[2].coords.x = pt.x + ((right.x + up.x) * decal->Size);
   decal->Vertex[2].coords.y = pt.y + ((right.y + up.y) * decal->Size);
   decal->Vertex[2].coords.z = pt.z + ((right.z + up.z) * decal->Size);
   decal->Vertex[3].coords.x = pt.x + ((-right.x + up.x) * decal->Size);
   decal->Vertex[3].coords.y = pt.y + ((-right.y + up.y) * decal->Size);
   decal->Vertex[3].coords.z = pt.z + ((-right.z + up.z) * decal->Size);
}

void RenderBulletDecal(DECAL decal)
{
    float mat_ambient[] = { 0.8, 0.8, 0.8, 1.0 };
    float mat_diffuse[] = { 0.8, 0.8, 0.8, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_ALPHA_TEST);
    glAlphaFunc(GL_GREATER, 0);

    glBindTexture(GL_TEXTURE_2D, texture[7].TexID);

    glBegin(GL_QUADS);
        glTexCoord2f(0.0f, 0.0f);
        glVertex3fv(&decal.Vertex[0].coords.x);
        glTexCoord2f(1.0f, 0.0f);
        glVertex3fv(&decal.Vertex[1].coords.x);
        glTexCoord2f(1.0f, 1.0f);
        glVertex3fv(&decal.Vertex[2].coords.x);
        glTexCoord2f(0.0f, 1.0f);
        glVertex3fv(&decal.Vertex[3].coords.x);
    glEnd();

    glDisable(GL_BLEND);
    glDisable(GL_ALPHA_TEST);
}

void RenderBurnDecal(DECAL decal)
{
    glEnable(GL_BLEND);
    glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA);
    glBindTexture(GL_TEXTURE_2D, texture[4].TexID);

    glBegin(GL_QUADS);
        glTexCoord2f(0.0f, 0.0f);
        glVertex3fv(&decal.Vertex[0].coords.x);
        glTexCoord2f(1.0f, 0.0f);
        glVertex3fv(&decal.Vertex[1].coords.x);
        glTexCoord2f(1.0f, 1.0f);
        glVertex3fv(&decal.Vertex[2].coords.x);
        glTexCoord2f(0.0f, 1.0f);
        glVertex3fv(&decal.Vertex[3].coords.x);
    glEnd();

    glDisable(GL_BLEND);
}

void DrawImpactFlash()
{
    float halfwidth = 1.0;
    float halfheight = 1.0;

    MATRIX mat;
    glGetFloatv(GL_MODELVIEW_MATRIX, mat.Element);

    VECTOR right;
    right.x = mat.Element[0];
    right.y = mat.Element[4];
    right.z = mat.Element[8];
    right.Normalize();
    right.x *= halfwidth;
    right.y *= halfwidth;
    right.z *= halfwidth;

    VECTOR up;
    up.x = mat.Element[1];
    up.y = mat.Element[5];
    up.z = mat.Element[9];
    up.Normalize();
    up.x *= halfheight;
    up.y *= halfheight;
    up.z *= halfheight;

    glEnable(GL_BLEND);
    glDepthMask(0);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glDisable(GL_LIGHTING);
//    glColor4f(1.0, 1.0, 1.0, 1.0);
    glDisable(GL_CULL_FACE);
    glBindTexture(GL_TEXTURE_2D, texture[6].TexID);
    glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 0.0f); glVertex3f(impactflashdecal.CollisionVector.x + (-right.x - up.x), impactflashdecal.CollisionVector.y + (-right.y - up.y), impactflashdecal.CollisionVector.z + (-right.z - up.z));
            glTexCoord2f(1.0f, 0.0f); glVertex3f(impactflashdecal.CollisionVector.x + (right.x - up.x), impactflashdecal.CollisionVector.y + (right.y - up.y), impactflashdecal.CollisionVector.z + (right.z - up.z));
            glTexCoord2f(1.0f, 1.0f); glVertex3f(impactflashdecal.CollisionVector.x + (right.x + up.x), impactflashdecal.CollisionVector.y + (right.y + up.y), impactflashdecal.CollisionVector.z + (right.z + up.z));
            glTexCoord2f(0.0f, 1.0f); glVertex3f(impactflashdecal.CollisionVector.x + (up.x - right.x), impactflashdecal.CollisionVector.y + (up.y - right.y), impactflashdecal.CollisionVector.z + (up.z - right.z));
    glEnd();
    glEnable(GL_DEPTH_TEST);
    glDepthMask(1);
    glDisable(GL_BLEND);
    glEnable(GL_LIGHTING);
    glEnable(GL_CULL_FACE);
}

void DrawMuzzleFlash()
{
    MATRIX Matrix;
    glPushMatrix();
    glTranslatef(muzzleflashdecal.Position.x,muzzleflashdecal.Position.y,muzzleflashdecal.Position.z);
    Matrix.QuatToMatrix(muzzleflashdecal.Orientation);
    glMultMatrixf(Matrix.Element);
    glTranslatef(0.0,-4.0,-10.0);

    glEnable(GL_BLEND);
    glDepthMask(0);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glDisable(GL_LIGHTING);
    glColor4f(1.0, 1.0, 1.0, 1.0);
    glBindTexture(GL_TEXTURE_2D, texture[5].TexID);
    glDisable(GL_CULL_FACE);

    glBegin(GL_QUADS);
        glTexCoord2f(0.0f, 0.0f);
        glVertex3f(-muzzleflashdecal.Size, -muzzleflashdecal.Size, 0.0);
        glTexCoord2f(1.0f, 0.0f);
        glVertex3f(muzzleflashdecal.Size, -muzzleflashdecal.Size, 0.0);
        glTexCoord2f(1.0f, 1.0f);
        glVertex3f(muzzleflashdecal.Size, muzzleflashdecal.Size, 0.0);
        glTexCoord2f(0.0f, 1.0f);
        glVertex3f(-muzzleflashdecal.Size, muzzleflashdecal.Size, 0.0);
    glEnd();

    glEnable(GL_CULL_FACE);
    glDepthMask(1);
    glDisable(GL_BLEND);
    glEnable(GL_LIGHTING);
    glPopMatrix();
}

void DrawDecals()
{
    // If the impact-flash decal is active then render it
    if (impactflashdecal.active)
    {
        impactflashdecal.active = false;
        DrawImpactFlash();
    }

    // If the muzzle-flash decal is active then render it
    if (muzzleflashdecal.active)
    {
        muzzleflashdecal.active = false;
        muzzleflashdecal.Position = camera[currentCamera].Position;
        muzzleflashdecal.Orientation = camera[currentCamera].Orientation;
        DrawMuzzleFlash();
    }
}

