#include <windows.h>
#include "polygon.h"
#include "tll.h"
#include "locmath.h"
#include "collision.h"
#include "bsp.h"


POLYGON::POLYGON()
{
    NextPoly = NULL;
    numVertices    = 0;
    Texture = 0;
}

POLYGON::~POLYGON()
{
}

bool POLYGON::IsLast()
{
    if (NextPoly == NULL)
        return true;
    else
        return false;
}

POLYGON* POLYGON::GetNext()
{
    if (!IsLast())
        return NextPoly;
    else
        return NULL;
}

void POLYGON::SetNext(POLYGON* Poly)
{
    if (IsLast())
    {
        NextPoly = Poly;
        return;
    }

    // Insert the given list
    POLYGON* TempPoly = Poly;

    while (!TempPoly->IsLast())
        TempPoly = TempPoly->GetNext();

    TempPoly->SetNext(NextPoly);
    NextPoly = Poly;
}

POLYGON* POLYGON::CopyList()
{
    POLYGON* TempPoly = new POLYGON;

    TempPoly->Texture = Texture;

    TempPoly->numVertices = numVertices;

    for (int loop = 0; loop < numVertices; loop++)
        TempPoly->Vertex[loop] = Vertex[loop];

    if (!IsLast())
        TempPoly->AddPolygon(NextPoly->CopyList());

    return TempPoly;
}

void POLYGON::AddPolygon(POLYGON* Poly)
{
    if (Poly != NULL)
    {
        if (IsLast())
        {
            NextPoly = Poly;
            return;
        }

        POLYGON* TempPoly = NextPoly;

        while (!TempPoly->IsLast())
            TempPoly = TempPoly->GetNext();

        TempPoly->NextPoly = Poly;
    }
}

int POLYGON::Compare(const POLYGON& Polygon)
{
    if (linkPosition < Polygon.linkPosition)
            return smaller;
      if (linkPosition > Polygon.linkPosition)
            return bigger;
      else
            return same;
}

VECTOR POLYGON::GetNormal()
{
        VECTOR temp, u, v;
          u = Vertex[1].coords - Vertex[0].coords;
          v = Vertex[2].coords - Vertex[0].coords;
          temp.x = (u.y*v.z)-(v.y*u.z);
          temp.y = (u.z*v.x)-(v.z*u.x);
          temp.z = (u.x*v.y)-(v.x*u.y);
        return temp;
}

void POLYGON::SetNormal()
{
    VECTOR u;
    VECTOR v;
    VERTEX tempVertex;
    u = Vertex[1].coords - Vertex[0].coords;
    v = Vertex[2].coords - Vertex[0].coords;
    tempVertex.normal.x = (u.y*v.z)-(v.y*u.z);
    tempVertex.normal.y = (u.z*v.x)-(v.z*u.x);
    tempVertex.normal.z = (u.x*v.y)-(v.x*u.y);
    for (int loop = 0; loop < maxPolygonVerts; loop++)
    {
        Vertex[loop].normal = tempVertex.normal;
    }
}

VECTOR GetEdgeIntersection(VECTOR point0, VECTOR point1, POLYGON planePolygon)
{
    VECTOR edge1, edge2, planeNormal, pointOnPlane, intersection, temp;
    float numerator, denominator, t;

    // get a point on the plane
    pointOnPlane = planePolygon.Vertex[0].coords;

    // get the splitting planes normal
    edge1 = planePolygon.Vertex[1].coords - planePolygon.Vertex[0].coords;
    edge2 = planePolygon.Vertex[2].coords - planePolygon.Vertex[0].coords;
    planeNormal = CrossVector(edge1, edge2);

// find edge intersection:
// intersection = p0 + (p1 - p0) * t
// where t = (planeNormal . (pointOnPlane - p0)) / (planeNormal . (p1 - p0))

    //planeNormal . (pointOnPlane - point0)
    temp = pointOnPlane - point0;
    numerator = DotProduct(planeNormal, temp);

    //planeNormal . (point1 - point0)
    temp = point1 - point0;
    denominator = DotProduct(planeNormal, temp);

    if (denominator)
        t = numerator / denominator;
    else
        t = 0.0;

    intersection.x = point0.x + temp.x * t;
    intersection.y = point0.y + temp.y * t;
    intersection.z = point0.z + temp.z * t;

    return intersection;
}

void DeleteList(POLYGON* Poly)
{
    if (Poly->IsLast())
    {
        delete Poly;
        return;
    }

    DeleteList(Poly->GetNext());
    delete Poly;
}

int SplitPolygon(POLYGON* PolygonToSplit, POLYGON inputplanePolygon, POLYGON* front, POLYGON* back)
{
    const int MaxVerts = 100;
    int numVerts = PolygonToSplit->numVertices;
    int count = 0, out_c = 0, in_c = 0, sideA, sideB, loop;
    VECTOR planeNormal, polysNormal, pointOnPlane, edge1, edge2, temp;
    VERTEX ptA, ptB, intersection, outpts[MaxVerts], inpts[MaxVerts];
    VERTEX texvert1, texvert2; // texture calculation variables
    VECTOR t1, t2;
    float scale;
    int PolygonNumber = PolygonToSplit->Vertex[0].polyNumber;
    int Removable = PolygonToSplit->removable;
    GLuint Tex = PolygonToSplit->Texture;

    // get a point on the plane
    pointOnPlane = inputplanePolygon.Vertex[0].coords;

    // get the splitting planes normal
    edge1 = inputplanePolygon.Vertex[1].coords - inputplanePolygon.Vertex[0].coords;
    edge2 = inputplanePolygon.Vertex[2].coords - inputplanePolygon.Vertex[0].coords;
    planeNormal = CrossVector(edge1, edge2);

    // get the normal of the portal to split
    edge1 = PolygonToSplit->Vertex[1].coords - PolygonToSplit->Vertex[0].coords;
    edge2 = PolygonToSplit->Vertex[2].coords - PolygonToSplit->Vertex[0].coords;
    polysNormal = CrossVector(edge1, edge2);

    // check if the polygon lies on the plane
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = PolygonToSplit->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
            count++;
        else
            break;
    }

    if (count == numVerts)
        return OnPartition;

    // find if all of the points are infront of or behind the plane
    int frontcount = 0, backcount = 0;
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = PolygonToSplit->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
        {
            frontcount++;
            backcount++;
        }
        else if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 1)
            frontcount++;
        else if (ClassifyPoint(temp, pointOnPlane, planeNormal) == -1)
            backcount++;
    }
    if (frontcount == numVerts)
            return Front;
    if (backcount == numVerts)
            return Back;

    // try to split the polygon
    ptA = PolygonToSplit->Vertex[numVerts - 1];
    temp = ptA.coords;
    sideA = ClassifyPoint(temp, pointOnPlane, planeNormal);
    for (int i = -1; ++i < numVerts;)
    {
        ptB = PolygonToSplit->Vertex[i];
        temp = ptB.coords;
        sideB = ClassifyPoint(temp, pointOnPlane, planeNormal);
        if (sideB > 0)
        {
            if (sideA < 0)
            {
                // find intersection
                edge1 = ptA.coords;
                edge2 = ptB.coords;

                temp = GetEdgeIntersection(edge1, edge2, inputplanePolygon);
                intersection.coords = temp;

                // find the new texture coordinates
                texvert1.coords = ptB.coords - ptA.coords;
                texvert2.coords = intersection.coords - ptA.coords;
                texvert1.u = ptA.u;
                texvert2.u = ptB.u;
                texvert1.v = ptA.v;
                texvert2.v = ptB.v;
                t1 = texvert1.coords;
                t2 = texvert2.coords;
                scale = sqrt(t2.x*t2.x+t2.y*t2.y+t2.z*t2.z)/sqrt(t1.x*t1.x+t1.y*t1.y+t1.z*t1.z);
                intersection.u = texvert1.u + (texvert2.u - texvert1.u) * scale;
                intersection.v = texvert1.v + (texvert2.v - texvert1.v) * scale;

                outpts[out_c++] = inpts[in_c++] = intersection;
            }
            inpts[in_c++] = ptB;
        }
        else if (sideB < 0)
        {
            if (sideA > 0)
            {
                // find intersection
                edge1 = ptA.coords;
                edge2 = ptB.coords;

                temp = GetEdgeIntersection(edge1, edge2, inputplanePolygon);
                intersection.coords = temp;

                // find the new texture coordinates
                texvert1.coords = ptB.coords - ptA.coords;
                texvert2.coords = intersection.coords - ptA.coords;
                texvert1.u = ptA.u;
                texvert2.u = ptB.u;
                texvert1.v = ptA.v;
                texvert2.v = ptB.v;
                t1 = texvert1.coords;
                t2 = texvert2.coords;
                scale = sqrt(t2.x*t2.x+t2.y*t2.y+t2.z*t2.z)/sqrt(t1.x*t1.x+t1.y*t1.y+t1.z*t1.z);
                intersection.u = texvert1.u + (texvert2.u - texvert1.u) * scale;
                intersection.v = texvert1.v + (texvert2.v - texvert1.v) * scale;

                outpts[out_c++] = inpts[in_c++] = intersection;
            }
            outpts[out_c++] = ptB;
        }
        else
            outpts[out_c++] = inpts[in_c++] = ptB;
        ptA = ptB;
        sideA = sideB;
    }

    if (out_c == 0 || in_c == 0)
    {
        int side;

        for (int loop = 0; loop < numVerts; loop++)
        {
            temp = PolygonToSplit->Vertex[loop].coords;
            side = ClassifyPoint(temp, pointOnPlane, planeNormal);
            if (side == 1)
                return Front;
            else if (side == -1)
                return Back;
        }
    }
    else
    {
        front->numVertices = in_c;
        back->numVertices = out_c;

        for (int loop = 0; loop < in_c; loop++)
        {
            front->Vertex[loop] = inpts[loop];
            front->Vertex[loop].polyNumber = PolygonNumber;
        }
        for (int loop = 0; loop < out_c; loop++)
        {
            back->Vertex[loop] = outpts[loop];
            back->Vertex[loop].polyNumber = PolygonNumber;
        }
        front->removable = Removable;
        front->Texture = Tex;
        front->SetNormal();
        back->removable = Removable;
        back->Texture = Tex;
        back->SetNormal();
        return PolygonWasSplit;
    }

    return 0;
}

/*
 This function only splits triangles, the function inputs the triangle to be 
 split and another triangle to be used for the splitting plane. 
 The parameter 'triangles' is a pointer to 3 triangles for output.
 The function splits the input triangle into either 0, 2 or 3 new triangles with
 recalculated texture coordinates.
 If the triangles pointer happens to be NULL then the function will just set the outputFlag and return.

 The return value will be either Front, Back, TwoFrontOneBack, OneFrontTwoBack or OneFrontOneBack.
 Front means that all points are infront of the plane or the triangle lies on the plane and faces the front.
 Back means that all points are behind of the plane or the triangle lies on the plane and faces the back.
 TwoFrontOneBack means that triangles 1 and 2 are infront of the plane and triangle 3 is behind.
 OneFrontTwoBack means that triangle 1 is infront of the plane and triangles 2 and 3 are behind.
 OneFrontOneBack means that triangle 1 is infront of the plane and triangle 2 is behind, triangle 3 is not used.
*/
int SplitTriangle(POLYGON triangleToSplit, POLYGON planeTriangle, POLYGON* triangles)
{
    VECTOR planeNormal, polysNormal, pointOnPlane, edge1, edge2, temp;
    VERTEX ptA, ptB, outpts[4], inpts[4], intersection;
    int count = 0, out_c = 0, in_c = 0, sideA, sideB, outputFlag;
    VERTEX texvert1, texvert2; // texture calculation variables
    VECTOR t1, t2;
    float scale;

    // get a point on the plane
    pointOnPlane = planeTriangle.Vertex[0].coords;

    // get the splitting planes normal
    edge1 = planeTriangle.Vertex[1].coords - planeTriangle.Vertex[0].coords;
    edge2 = planeTriangle.Vertex[2].coords - planeTriangle.Vertex[0].coords;
    planeNormal = CrossVector(edge1, edge2);

    // get the normal of the triangle to split
    edge1 = triangleToSplit.Vertex[1].coords - triangleToSplit.Vertex[0].coords;
    edge2 = triangleToSplit.Vertex[2].coords - triangleToSplit.Vertex[0].coords;
    polysNormal = CrossVector(edge1, edge2);

    // check if the triangle lies on the plane
    for (int loop = 0; loop < 3; loop++)
    {
        temp = triangleToSplit.Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
            count++;
        else
            break;
    }
    if (count == 3)
    {
        if (ClassifyPoint(polysNormal, pointOnPlane, planeNormal) == 1)
            return Front;
        if (ClassifyPoint(polysNormal, pointOnPlane, planeNormal) == -1)
            return Back;
    }

    // find if all of the points are infront of or behind the plane
    int frontcount = 0, backcount = 0;
    for (int loop = 0; loop < 3; loop++)
    {
        temp = triangleToSplit.Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
        {
            frontcount++;
            backcount++;
        }
        else if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 1)
            frontcount++;
        else if (ClassifyPoint(temp, pointOnPlane, planeNormal) == -1)
            backcount++;
    }
    if (frontcount == 3)
            return Front;
    if (backcount == 3)
            return Back;

    // try to split the triangle
    ptA = triangleToSplit.Vertex[2];
    temp = ptA.coords;
    sideA = ClassifyPoint(temp, pointOnPlane, planeNormal);
    for (int i = -1; ++i < 3;)
    {
        ptB = triangleToSplit.Vertex[i];
        temp = ptB.coords;
        sideB = ClassifyPoint(temp, pointOnPlane, planeNormal);
        if (sideB > 0)
        {
            if (sideA < 0)
            {
                // find intersection
                edge1 = ptA.coords;
                edge2 = ptB.coords;

                temp = GetEdgeIntersection(edge1, edge2, planeTriangle);
                intersection.coords = temp;

                // find the new texture coordinates
                texvert1.coords = ptB.coords - ptA.coords;
                texvert2.coords = intersection.coords - ptA.coords;
                texvert1.u = ptA.u;
                texvert2.u = ptB.u;
                texvert1.v = ptA.v;
                texvert2.v = ptB.v;
                t1 = texvert1.coords;
                t2 = texvert2.coords;
                scale = sqrt(t2.x*t2.x+t2.y*t2.y+t2.z*t2.z)/sqrt(t1.x*t1.x+t1.y*t1.y+t1.z*t1.z);
                intersection.u = texvert1.u + (texvert2.u - texvert1.u) * scale;
                intersection.v = texvert1.v + (texvert2.v - texvert1.v) * scale;

                outpts[out_c++] = inpts[in_c++] = intersection;
            }
            inpts[in_c++] = ptB;
        }
        else if (sideB < 0)
        {
            if (sideA > 0)
            {
                // find intersection
                edge1 = ptA.coords;
                edge2 = ptB.coords;

                temp = GetEdgeIntersection(edge1, edge2, planeTriangle);
                intersection.coords = temp;

                // find the new texture coordinates
                texvert1.coords = ptB.coords - ptA.coords;
                texvert2.coords = intersection.coords - ptA.coords;
                texvert1.u = ptA.u;
                texvert2.u = ptB.u;
                texvert1.v = ptA.v;
                texvert2.v = ptB.v;
                t1 = texvert1.coords;
                t2 = texvert2.coords;
                scale = sqrt(t2.x*t2.x+t2.y*t2.y+t2.z*t2.z)/sqrt(t1.x*t1.x+t1.y*t1.y+t1.z*t1.z);
                intersection.u = texvert1.u + (texvert2.u - texvert1.u) * scale;
                intersection.v = texvert1.v + (texvert2.v - texvert1.v) * scale;

                outpts[out_c++] = inpts[in_c++] = intersection;
            }
            outpts[out_c++] = ptB;
        }
        else
            outpts[out_c++] = inpts[in_c++] = ptB;
            ptA = ptB;
            sideA = sideB;
    }

    if (in_c == 4)          // two triangles are infront, one behind
    {
        outputFlag = TwoFrontOneBack;
        if (triangles)
        {
            triangles[0].Vertex[0] = inpts[0];
            triangles[0].Vertex[1] = inpts[1];
            triangles[0].Vertex[2] = inpts[2];
            triangles[0].numVertices = 3;
            triangles[0].SetNormal();
            triangles[1].Vertex[0] = inpts[0];
            triangles[1].Vertex[1] = inpts[2];
            triangles[1].Vertex[2] = inpts[3];
            triangles[1].numVertices = 3;
            triangles[1].SetNormal();
            triangles[2].Vertex[0] = outpts[0];
            triangles[2].Vertex[1] = outpts[1];
            triangles[2].Vertex[2] = outpts[2];
            triangles[2].numVertices = 3;
            triangles[2].SetNormal();
        }
    }
    else if (out_c == 4)    // one triangle is infront, two behind
    {
        outputFlag = OneFrontTwoBack;
        if (triangles)
        {
            triangles[0].Vertex[0] = inpts[0];
            triangles[0].Vertex[1] = inpts[1];
            triangles[0].Vertex[2] = inpts[2];
            triangles[0].numVertices = 3;
            triangles[0].SetNormal();
            triangles[1].Vertex[0] = outpts[0];
            triangles[1].Vertex[1] = outpts[1];
            triangles[1].Vertex[2] = outpts[2];
            triangles[1].numVertices = 3;
            triangles[1].SetNormal();
            triangles[2].Vertex[0] = outpts[0];
            triangles[2].Vertex[1] = outpts[2];
            triangles[2].Vertex[2] = outpts[3];
            triangles[2].numVertices = 3;
            triangles[2].SetNormal();
        }
    }
    else if (in_c == 3 && out_c == 3)  // plane bisects the triangle
    {
        outputFlag = OneFrontOneBack;
        if (triangles)
        {
            triangles[0].Vertex[0] = inpts[0];
            triangles[0].Vertex[1] = inpts[1];
            triangles[0].Vertex[2] = inpts[2];
            triangles[0].numVertices = 3;
            triangles[0].SetNormal();
            triangles[1].Vertex[0] = outpts[0];
            triangles[1].Vertex[1] = outpts[1];
            triangles[1].Vertex[2] = outpts[2];
            triangles[1].numVertices = 3;
            triangles[1].SetNormal();
        }
    }
    else // then triangle must be totally infront of or behind the plane
    {
        int side;

        for (int loop = 0; loop < 3; loop++)
        {
            temp = triangleToSplit.Vertex[loop].coords;
            side = ClassifyPoint(temp, pointOnPlane, planeNormal);
            if (side == 1)
            {
                outputFlag = Front;
                break;
            }
            else if (side == -1)
            {
                outputFlag = Back;
                break;
            }
        }
    }
    return outputFlag;
}

int ClassifyPolygon(POLYGON* Polygon, POLYGON planePolygon)
{
    int numVerts = Polygon->numVertices;
    int count = 0;
    VECTOR planeNormal, polysNormal, pointOnPlane, edge1, edge2, temp;

    // get a point on the plane
    pointOnPlane = planePolygon.Vertex[0].coords;

    // get the planes normal
    edge1 = planePolygon.Vertex[1].coords - planePolygon.Vertex[0].coords;
    edge2 = planePolygon.Vertex[2].coords - planePolygon.Vertex[0].coords;
    planeNormal = CrossVector(edge1, edge2);

    // get the normal of the Polygon
    edge1 = Polygon->Vertex[1].coords - Polygon->Vertex[0].coords;
    edge2 = Polygon->Vertex[2].coords - Polygon->Vertex[0].coords;
    polysNormal = CrossVector(edge1, edge2);

    // check if the polygon lies on the plane
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = Polygon->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
            count++;
        else
            break;
    }
    if (count == numVerts)
            return OnPartition;

    // find if all of the points are infront of or behind the plane
    int result, frontcount = 0, backcount = 0;
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = Polygon->Vertex[loop].coords;
        result = ClassifyPoint(temp, pointOnPlane, planeNormal);
        if (result == 0)
        {
            frontcount++;
            backcount++;
        }
        else
            if (result == 1)
                frontcount++;
        else
            if (result == -1)
                backcount++;
    }
    if (frontcount == numVerts)
            return Front;
    if (backcount == numVerts)
            return Back;

    return PolygonWasSplit;
}

void InvertPolygon(POLYGON* Polygon)
{
    POLYGON temppolygon = *Polygon;
    for (int loop = 0; loop < Polygon->numVertices; loop++)
        Polygon->Vertex[loop] = temppolygon.Vertex[(Polygon->numVertices - 1) - loop];
    Polygon->SetNormal();
    return;
}
