// Polygon Class    by Alan Baylis 2002

#ifndef Polygon_H
#define Polygon_H

#include "vector.h"
#include "vertex.h"

const int maxPolygonVerts = 20;

struct BSP_node;

class POLYGON
{
    public:
        POLYGON();
          ~POLYGON();

        // methods required for linked list 
        int Compare(const POLYGON& Polygon);
        int GetMyPosition() const {return linkPosition;}
        void SetMyPosition(int newPosition) {linkPosition = newPosition;}
        int linkPosition;

        // member variables and methods required to link polygons
        POLYGON* NextPoly;
        bool IsLast();        
        POLYGON* GetNext();
        void AddPolygon(POLYGON* Poly);
        void SetNext(POLYGON* Poly);
        POLYGON* CopyList();

        VECTOR GetNormal();
        void SetNormal();

        int id;
        int numVertices;
        int removable;
        unsigned int Texture;
        float Scale[2];
        float Shift[2];
        float Rotate;
        VERTEX Vertex[maxPolygonVerts];

        // operator overloading
        const bool POLYGON::operator == (const POLYGON &Poly) const
        {
            float PolyDistance = -(Poly.Vertex[0].normal.x * Poly.Vertex[0].coords.x + Poly.Vertex[0].normal.y * Poly.Vertex[0].coords.y + Poly.Vertex[0].normal.z * Poly.Vertex[0].coords.z);
            float ThisDistance = -(Vertex[0].normal.x * Vertex[0].coords.x + Vertex[0].normal.y * Vertex[0].coords.y + Vertex[0].normal.z * Vertex[0].coords.z);

            if (numVertices == Poly.numVertices)
            {
                if (PolyDistance == ThisDistance)
                {
                    if (Vertex[0].normal == Poly.Vertex[0].normal)
                    {
                        for (int loop = 0; loop < numVertices; loop++)
                        {
                            if (Vertex[loop].u != Poly.Vertex[loop].u)
                                return false;
                            if (Vertex[loop].v != Poly.Vertex[loop].v)
                                return false;
                        }

                        if (Texture == Poly.Texture)
                            return true;
                    }
                }
            }
            return false;
        }
};

void DeleteList(POLYGON* Poly);
int SplitPolygon(POLYGON* PolygonToSplit, POLYGON inputplanePolygon, POLYGON* front, POLYGON* back);
int SplitTriangle(POLYGON triangleToSplit, POLYGON planeTriangle, POLYGON* triangles);
int ClassifyPolygon(POLYGON* Polygon, POLYGON planePolygon);
void InvertPolygon(POLYGON* Polygon);
VECTOR GetEdgeIntersection(VECTOR point0, VECTOR point1, POLYGON planePolygon);

#endif
