#include <windows.h>
#include "portal.h"
#include "locmath.h"
#include "listnode.h"
#include "collision.h"
#include "general.h"
#include "bsp.h"
#include "tll.h"
#include "plane.h"


extern float Min_X, Min_Y, Min_Z, Max_X, Max_Y, Max_Z; // The bounding box variables
extern int numpartitions;
extern BSP_node* listnode;
extern LinkedList<BSP_node> LeafList;
extern LinkedList<BSP_node> PartitionList;
extern LinkedList<PORTAL> PortalList;
extern int numportals;
extern PORTAL* portal;
extern BSP_node* root;

PORTAL::PORTAL()
{
}

PORTAL::~PORTAL()
{
}

int PORTAL::Compare(const PORTAL& Portal)
{
    if (linkPosition < Portal.linkPosition)
            return smaller;
      if (linkPosition > Portal.linkPosition)
            return bigger;
      else
            return same;
}

VECTOR PORTAL::GetNormal()
{
        VECTOR temp;
        VECTOR u;
        VECTOR v;
          u = Vertex[1].coords - Vertex[0].coords;
          v = Vertex[2].coords - Vertex[0].coords;
          temp.x = (u.y*v.z)-(v.y*u.z);
          temp.y = (u.z*v.x)-(v.z*u.x);
          temp.z = (u.x*v.y)-(v.x*u.y);
        return temp;
}

void PORTAL::SetNormal()
{
        VECTOR u;
        VECTOR v;
          u = Vertex[1].coords - Vertex[0].coords;
          v = Vertex[2].coords - Vertex[0].coords;
          Vertex[0].normal.x = (u.y*v.z)-(v.y*u.z);
          Vertex[0].normal.y = (u.z*v.x)-(v.z*u.x);
          Vertex[0].normal.z = (u.x*v.y)-(v.x*u.y);
        Vertex[1].normal = Vertex[0].normal;
          Vertex[2].normal = Vertex[0].normal;
}

PORTAL* CopyPortal(PORTAL* portal)
{
    PORTAL* tempportal = new PORTAL;
    *tempportal = *portal;
    tempportal->Vertex = new VERTEX[portal->numVertices];
    for (int loop = 0; loop < portal->numVertices; loop++)
        tempportal->Vertex[loop] = portal->Vertex[loop];
    return tempportal;
}

void CreateLargePortal(POLYGON splittingPolygon, PORTAL* largePortal)
{
// Make large portal using planar mapping
    largePortal->numVertices = 4;
    largePortal->Vertex = new VERTEX[4];
    // find the primary axis plane
    int flag = 0;
    VECTOR poly_normal = splittingPolygon.GetNormal();
    poly_normal.Normalize();

    if (fabs(poly_normal.x) > fabs(poly_normal.y) && fabs(poly_normal.x) > fabs(poly_normal.z))
    {
        flag = 1;
        largePortal->Vertex[0].coords.y = Min_Y;
        largePortal->Vertex[0].coords.z = Max_Z;
        largePortal->Vertex[1].coords.y = Min_Y;
        largePortal->Vertex[1].coords.z = Min_Z;
        largePortal->Vertex[2].coords.y = Max_Y;
        largePortal->Vertex[2].coords.z = Min_Z;
        largePortal->Vertex[3].coords.y = Max_Y;
        largePortal->Vertex[3].coords.z = Max_Z;
    }
    else if (fabs(poly_normal.y) > fabs(poly_normal.x) && fabs(poly_normal.y) > fabs(poly_normal.z))
    {
        flag = 2;
        largePortal->Vertex[0].coords.x = Min_X;
        largePortal->Vertex[0].coords.z = Max_Z;
        largePortal->Vertex[1].coords.x = Max_X;
        largePortal->Vertex[1].coords.z = Max_Z;
        largePortal->Vertex[2].coords.x = Max_X;
        largePortal->Vertex[2].coords.z = Min_Z;
        largePortal->Vertex[3].coords.x = Min_X;
        largePortal->Vertex[3].coords.z = Min_Z;
    }
    else
    {
        flag = 3;
        largePortal->Vertex[0].coords.x = Min_X;
        largePortal->Vertex[0].coords.y = Min_Y;
        largePortal->Vertex[1].coords.x = Max_X;
        largePortal->Vertex[1].coords.y = Min_Y;
        largePortal->Vertex[2].coords.x = Max_X;
        largePortal->Vertex[2].coords.y = Max_Y;
        largePortal->Vertex[3].coords.x = Min_X;
        largePortal->Vertex[3].coords.y = Max_Y;
    }

    float X, Y, Z;
    float Distance = - (poly_normal.x * splittingPolygon.Vertex[0].coords.x + poly_normal.y * splittingPolygon.Vertex[0].coords.y + poly_normal.z * splittingPolygon.Vertex[0].coords.z);
    switch (flag)
    {
        case 1: //YZ Plane
            X = - (poly_normal.y * Min_Y + poly_normal.z * Max_Z + Distance ) / poly_normal.x;
            largePortal->Vertex[0].coords.x = X;
            X = - (poly_normal.y * Min_Y + poly_normal.z * Min_Z + Distance ) / poly_normal.x;
            largePortal->Vertex[1].coords.x = X;
            X = - (poly_normal.y * Max_Y + poly_normal.z * Min_Z + Distance ) / poly_normal.x;
            largePortal->Vertex[2].coords.x = X;
            X = - (poly_normal.y * Max_Y + poly_normal.z * Max_Z + Distance ) / poly_normal.x;
            largePortal->Vertex[3].coords.x = X;
        break;
        case 2: //XZ Plane
            Y = - (poly_normal.x * Min_X + poly_normal.z * Max_Z + Distance ) / poly_normal.y;
            largePortal->Vertex[0].coords.y = Y;
            Y = - (poly_normal.x * Max_X + poly_normal.z * Max_Z + Distance ) / poly_normal.y;
            largePortal->Vertex[1].coords.y = Y;
            Y = - (poly_normal.x * Max_X + poly_normal.z * Min_Z + Distance ) / poly_normal.y;
            largePortal->Vertex[2].coords.y = Y;
            Y = - (poly_normal.x * Min_X + poly_normal.z * Min_Z + Distance ) / poly_normal.y;
            largePortal->Vertex[3].coords.y = Y;
        break;
        case 3: //XY Plane
            Z = - (poly_normal.x * Min_X + poly_normal.y * Min_Y + Distance ) / poly_normal.z;
            largePortal->Vertex[0].coords.z = Z;
            Z = - (poly_normal.x * Max_X + poly_normal.y * Min_Y + Distance ) / poly_normal.z;
            largePortal->Vertex[1].coords.z = Z;
            Z = - (poly_normal.x * Max_X + poly_normal.y * Max_Y + Distance ) / poly_normal.z;
            largePortal->Vertex[2].coords.z = Z;
            Z = - (poly_normal.x * Min_X + poly_normal.y * Max_Y + Distance ) / poly_normal.z;
            largePortal->Vertex[3].coords.z = Z;
        break;
    }
    largePortal->SetNormal();
}

int SplitPortal(PORTAL* portalToSplit, POLYGON planePolygon, PORTAL* front, PORTAL* back)
{
    const int MaxVerts = 100;
    int numVerts = portalToSplit->numVertices;
    int count = 0, out_c = 0, in_c = 0, sideA, sideB, loop;
    VECTOR planeNormal, polysNormal, pointOnPlane, edge1, edge2, temp;
    VERTEX ptA, ptB, intersection, outpts[MaxVerts], inpts[MaxVerts];

    // get a point on the plane
    pointOnPlane = planePolygon.Vertex[0].coords;

    // get the splitting planes normal
    edge1 = planePolygon.Vertex[1].coords - planePolygon.Vertex[0].coords;
    edge2 = planePolygon.Vertex[2].coords - planePolygon.Vertex[0].coords;
    planeNormal = CrossVector(edge1, edge2);

    // get the normal of the portal to split
    edge1 = portalToSplit->Vertex[1].coords - portalToSplit->Vertex[0].coords;
    edge2 = portalToSplit->Vertex[2].coords - portalToSplit->Vertex[0].coords;
    polysNormal = CrossVector(edge1, edge2);

    // check if the portal lies on the plane
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = portalToSplit->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
            count++;
        else
            break;
    }
    if (count == numVerts)
    {
        if (ClassifyPoint(polysNormal, pointOnPlane, planeNormal) == 1)
            return Front;
        if (ClassifyPoint(polysNormal, pointOnPlane, planeNormal) == -1)
            return Back;
    }

    // find if all of the points are infront of or behind the plane
    int frontcount = 0, backcount = 0;
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = portalToSplit->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
        {
            frontcount++;
            backcount++;
        }
        else if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 1)
            frontcount++;
        else if (ClassifyPoint(temp, pointOnPlane, planeNormal) == -1)
            backcount++;
    }
    if (frontcount == numVerts)
            return Front;
    if (backcount == numVerts)
            return Back;

    // try to split the portal
    ptA = portalToSplit->Vertex[numVerts - 1];
    temp = ptA.coords;
    sideA = ClassifyPoint(temp, pointOnPlane, planeNormal);
    for (int i = -1; ++i < numVerts;)
    {
        ptB = portalToSplit->Vertex[i];
        temp = ptB.coords;
        sideB = ClassifyPoint(temp, pointOnPlane, planeNormal);
        if (sideB > 0)
        {
            if (sideA < 0)
            {
                // find intersection
                edge1 = ptA.coords;
                edge2 = ptB.coords;

                temp = GetEdgeIntersection(edge1, edge2, planePolygon);
                intersection.coords = temp;

                outpts[out_c++] = inpts[in_c++] = intersection;
            }
            inpts[in_c++] = ptB;
        }
        else if (sideB < 0)
        {
            if (sideA > 0)
            {
                // find intersection
                edge1 = ptA.coords;
                edge2 = ptB.coords;

                temp = GetEdgeIntersection(edge1, edge2, planePolygon);
                intersection.coords = temp;

                outpts[out_c++] = inpts[in_c++] = intersection;
            }
            outpts[out_c++] = ptB;
        }
        else
            outpts[out_c++] = inpts[in_c++] = ptB;
        ptA = ptB;
        sideA = sideB;
    }

    if (out_c == 0 || in_c == 0)
    {
        int side;

        for (int loop = 0; loop < numVerts; loop++)
        {
            temp = portalToSplit->Vertex[loop].coords;
            side = ClassifyPoint(temp, pointOnPlane, planeNormal);
            if (side == 1)
                return Front;
            else if (side == -1)
                return Back;
        }
    }
    else
    {
        front->Vertex = new VERTEX[in_c];
        back->Vertex = new VERTEX[out_c];
        front->numVertices = in_c;
        back->numVertices = out_c;

        for (loop = 0; loop < in_c; loop++)
            front->Vertex[loop] = inpts[loop];
        for (loop = 0; loop < out_c; loop++)
            back->Vertex[loop] = outpts[loop];
        return PortalWasSplit;
    }
    return 0;
}

int SplitPortal(PORTAL* portalToSplit, PLANE plane, VECTOR pointOnPlane, PORTAL* front, PORTAL* back)
{
    const int MaxVerts = 100;
    int numVerts = portalToSplit->numVertices;
    int count = 0, out_c = 0, in_c = 0, sideA, sideB, loop;
    VECTOR planeNormal, polysNormal, edge1, edge2, temp;
    VERTEX ptA, ptB, intersection, outpts[MaxVerts], inpts[MaxVerts];

    // get the splitting planes normal
    planeNormal = plane.normal;

    // get the normal of the portal to split
    edge1 = portalToSplit->Vertex[1].coords - portalToSplit->Vertex[0].coords;
    edge2 = portalToSplit->Vertex[2].coords - portalToSplit->Vertex[0].coords;
    polysNormal = CrossVector(edge1, edge2);

    // check if the portal lies on the plane
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = portalToSplit->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
            count++;
        else
            break;
    }
    if (count == numVerts)
    {
        if (ClassifyPoint(polysNormal, pointOnPlane, planeNormal) == 1)
            return Front;
        if (ClassifyPoint(polysNormal, pointOnPlane, planeNormal) == -1)
            return Back;
    }

    // find if all of the points are infront of or behind the plane
    int frontcount = 0, backcount = 0;
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = portalToSplit->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
        {
            frontcount++;
            backcount++;
        }
        else if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 1)
            frontcount++;
        else if (ClassifyPoint(temp, pointOnPlane, planeNormal) == -1)
            backcount++;
    }
    if (frontcount == numVerts)
            return Front;
    if (backcount == numVerts)
            return Back;

    // try to split the portal
    ptA = portalToSplit->Vertex[numVerts - 1];
    temp = ptA.coords;
    sideA = ClassifyPoint(temp, pointOnPlane, planeNormal);
    for (int i = -1; ++i < numVerts;)
    {
        ptB = portalToSplit->Vertex[i];
        temp = ptB.coords;
        sideB = ClassifyPoint(temp, pointOnPlane, planeNormal);
        if (sideB > 0)
        {
            if (sideA < 0)
            {
                // find intersection
                edge1 = ptA.coords;
                edge2 = ptB.coords;

                temp = GetEdgeIntersection(edge1, edge2, plane, pointOnPlane);
                intersection.coords = temp;

                outpts[out_c++] = inpts[in_c++] = intersection;
            }
            inpts[in_c++] = ptB;
        }
        else if (sideB < 0)
        {
            if (sideA > 0)
            {
                // find intersection
                edge1 = ptA.coords;
                edge2 = ptB.coords;

                temp = GetEdgeIntersection(edge1, edge2, plane, pointOnPlane);
                intersection.coords = temp;

                outpts[out_c++] = inpts[in_c++] = intersection;
            }
            outpts[out_c++] = ptB;
        }
        else
            outpts[out_c++] = inpts[in_c++] = ptB;
        ptA = ptB;
        sideA = sideB;
    }

    if (out_c == 0 || in_c == 0)
    {
        int side;

        for (int loop = 0; loop < numVerts; loop++)
        {
            temp = portalToSplit->Vertex[loop].coords;
            side = ClassifyPoint(temp, pointOnPlane, planeNormal);
            if (side == 1)
                return Front;
            else if (side == -1)
                return Back;
        }
    }
    else
    {
        front->Vertex = new VERTEX[in_c];
        back->Vertex = new VERTEX[out_c];
        front->numVertices = in_c;
        back->numVertices = out_c;

        for (loop = 0; loop < in_c; loop++)
            front->Vertex[loop] = inpts[loop];
        for (loop = 0; loop < out_c; loop++)
            back->Vertex[loop] = outpts[loop];
        return PortalWasSplit;
    }
    return 0;
}

void MakePortalList()
{
    // Create the large portals
    for (int loop = 1; loop <= PartitionList.numObjects; loop++)
    {
        listnode = PartitionList.Get(loop);
        portal = new PORTAL;
        portal->linkPosition = loop;
        portal->PartitionNodeId = loop;
        CreateLargePortal(listnode->partition, portal);
        PortalList.Insert(portal);
    }

    // Split the large portals into potential portals
    PortalList.numObjects = PartitionList.numObjects;
    int result, portalToSplit;
    PORTAL* frontportal;
    PORTAL* backportal;
    for (int partition = 1; partition <= numpartitions; partition++)
    {
        listnode = PartitionList.Get(partition);
        portalToSplit = PortalList.numObjects;
        while (portalToSplit > 0)
        {
            frontportal = new PORTAL;
            backportal = new PORTAL;

            portal = PortalList.Get(portalToSplit);
            result = SplitPortal(portal, listnode->partition, frontportal, backportal);

            if (result == PortalWasSplit)
            {
                frontportal->linkPosition = ++PortalList.numObjects;
                frontportal->PartitionNodeId = portal->PartitionNodeId;
                PortalList.Insert(frontportal);
                backportal->linkPosition = ++PortalList.numObjects;
                backportal->PartitionNodeId = portal->PartitionNodeId;
                PortalList.Insert(backportal);

                delete[] portal->Vertex;
                PortalList.Delete(portalToSplit);
                PortalList.numObjects--;
            }
            else
            {
                delete frontportal;
                delete backportal;
            }
            portalToSplit--;
        }
    }
    // Loop through the portals and assign a unique id
    for (int loop = 1; loop <= PortalList.numObjects; loop++)
    {
        portal = PortalList.Get(loop);
        portal->PortalId = loop;
    }
}

int ClassifyPortal(PORTAL* portal, POLYGON planePolygon)
{
    int numVerts = portal->numVertices;
    int count = 0;
    VECTOR planeNormal, polysNormal, pointOnPlane, edge1, edge2, temp;

    // get a point on the plane
    pointOnPlane = planePolygon.Vertex[0].coords;

    // get the planes normal
    edge1 = planePolygon.Vertex[1].coords - planePolygon.Vertex[0].coords;
    edge2 = planePolygon.Vertex[2].coords - planePolygon.Vertex[0].coords;
    planeNormal = CrossVector(edge1, edge2);

    // get the normal of the portal
    edge1 = portal->Vertex[1].coords - portal->Vertex[0].coords;
    edge2 = portal->Vertex[2].coords - portal->Vertex[0].coords;
    polysNormal = CrossVector(edge1, edge2);

    // check if the portal lies on the plane
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = portal->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
            count++;
        else
            break;
    }
    if (count == numVerts)
            return OnPartition;

    // find if all of the points are infront of or behind the plane
    int result, frontcount = 0, backcount = 0;
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = portal->Vertex[loop].coords;
        result = ClassifyPoint(temp, pointOnPlane, planeNormal);
        if (result == 0)
        {
            frontcount++;
            backcount++;
        }
        else
            if (result == 1)
                frontcount++;
        else
            if (result == -1)
                backcount++;
    }
    if (frontcount == numVerts)
            return Front;
    if (backcount == numVerts)
            return Back;

    return 0;
}

int ClassifyInvertedPortal(PORTAL* portal, POLYGON planePolygon)
{
    int numVerts = portal->numVertices;
    int count = 0;
    VECTOR planeNormal, polysNormal, pointOnPlane, edge1, edge2, temp;

    // get a point on the plane
    pointOnPlane = planePolygon.Vertex[0].coords;

    // get the planes normal
    edge1 = planePolygon.Vertex[1].coords - planePolygon.Vertex[0].coords;
    edge2 = planePolygon.Vertex[2].coords - planePolygon.Vertex[0].coords;
    planeNormal = CrossVector(edge1, edge2);

    // get the normal of the portal
    edge1 = portal->Vertex[1].coords - portal->Vertex[0].coords;
    edge2 = portal->Vertex[2].coords - portal->Vertex[0].coords;
    polysNormal = CrossVector(edge1, edge2);
    polysNormal.x *= -1;
    polysNormal.y *= -1;
    polysNormal.z *= -1;

    // check if the portal lies on the plane
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = portal->Vertex[loop].coords;
        if (ClassifyPoint(temp, pointOnPlane, planeNormal) == 0)
            count++;
        else
            break;
    }
    if (count == numVerts)
    {
        if (ClassifyPoint(polysNormal, pointOnPlane, planeNormal) == 1)
            return Front;
        if (ClassifyPoint(polysNormal, pointOnPlane, planeNormal) == -1)
            return Back;
    }

    // find if all of the points are infront of or behind the plane
    int result, frontcount = 0, backcount = 0;
    for (int loop = 0; loop < numVerts; loop++)
    {
        temp = portal->Vertex[loop].coords;
        result = ClassifyPoint(temp, pointOnPlane, planeNormal);
        if (result == 0)
        {
            frontcount++;
            backcount++;
        }
        else
            if (result == 1)
                frontcount++;
        else
            if (result == -1)
                backcount++;
    }
    if (frontcount == numVerts)
            return Front;
    if (backcount == numVerts)
            return Back;

    return 0;
}

void InvertPortal(PORTAL* portal)
{
    PORTAL* tempportal = CopyPortal(portal);
    int numverts = portal->numVertices;
    for (int loop = 0; loop < numverts; loop++)
        portal->Vertex[loop] = tempportal->Vertex[(numverts - 1) - loop];
    return;
}

void InvertPortals(BSP_node* node)
{
    VECTOR planeNormal, polysNormal, pointOnPlane, edge1, edge2, temp;
    int flag;
    for (int loop = 1; loop <= node->numportals; loop++)
    {
        PORTAL* tempportal = node->portallist.Get(loop);
        // get a point on the plane
        pointOnPlane = tempportal->Vertex[0].coords;

        // get the normal of the portals plane
        edge1 = tempportal->Vertex[1].coords - tempportal->Vertex[0].coords;
        edge2 = tempportal->Vertex[2].coords - tempportal->Vertex[0].coords;
        planeNormal = CrossVector(edge1, edge2);

        flag = 0;
        for (int polygon = 0; polygon < node->numpolys; polygon++)
        {
            for (int vert = 0; vert < 3; vert++)
            {
                temp = node->nodepolylist[polygon].Vertex[vert].coords;

                flag = ClassifyPoint(temp, pointOnPlane, planeNormal);
                if (flag == -1)
                {
                    InvertPortal(tempportal);
                }
                if (flag == 1 || flag == -1)
                {
                    if (tempportal->frontleaf->nodeid != node->nodeid)
                    {
                        BSP_node* tempnode = tempportal->backleaf;
                        tempportal->backleaf = tempportal->frontleaf;
                        tempportal->frontleaf = tempnode;
                    }
                    break;
                }
            }
            if (flag == 1 || flag == -1)
                break;
        }
    }
}

void AddPortal(PORTAL* thisportal, BSP_node* node)
{
    if (node->leaf == true)
    {
        node->numportals++;
        thisportal->linkPosition = node->numportals;
        node->portallist.Insert(thisportal);
        return;
    }

    int side = ClassifyPortal(thisportal, node->partition);
    if (side == Front)
    {
        AddPortal(thisportal, node->frontnode);
    }
    if (side == Back)
    {
        AddPortal(thisportal, node->backnode);
    }
    if (side == OnPartition)
    {
        AddPortal(thisportal, node->frontnode);
        PORTAL* tempportal = CopyPortal(thisportal);
        AddPortal(tempportal, node->backnode);
    }
    return;
}

void AddPortalsToLeaves(BSP_node* node)
{
    for (int loop = 1; loop <= PortalList.numObjects; loop++)
    {
        PORTAL* tempportal = CopyPortal(PortalList.Get(loop));
        AddPortal(tempportal, node);
    }
}

int ClipPortalToFrontLeaf(PORTAL* portal)
{
    int result, returnvalue = 0;
    for (int polygon = 0; polygon < portal->frontleaf->numpolys; polygon++)
    {
        PORTAL* front = new PORTAL;
        PORTAL* back = new PORTAL;
        result = SplitPortal(portal, portal->frontleaf->nodepolylist[polygon], front, back);

        if (result == PortalWasSplit)
        {
            returnvalue = PortalWasSplit;
            delete[] back->Vertex;
            delete back;
            delete[] portal->Vertex;
            portal->numVertices = front->numVertices;
            portal->Vertex = front->Vertex;
            delete front;
        }

        else
        {
            delete back;
            delete front;
        }
    }
    return returnvalue;
}

int ClipPortalToBackLeaf(PORTAL* portal)
{
    int result, returnvalue = 0;
    for (int polygon = 0; polygon < portal->backleaf->numpolys; polygon++)
    {
        PORTAL* front = new PORTAL;
        PORTAL* back = new PORTAL;
        result = SplitPortal(portal, portal->backleaf->nodepolylist[polygon], front, back);

        if (result == PortalWasSplit)
        {
            returnvalue = PortalWasSplit;
            delete[] back->Vertex;
            delete back;
            delete[] portal->Vertex;
            portal->numVertices = front->numVertices;
            portal->Vertex = front->Vertex;
            delete front;
        }
        else
        {
            delete back;
            delete front;
        }
    }
    return returnvalue;
}

void CheckForSinglePortals(BSP_node* node, BSP_node* originalnode, PORTAL* portal, int* flag)
{
    PORTAL* tempportal;
    if (node->leaf == true)
    {
        if (node->nodeid != originalnode->nodeid)
        {
            for (int portalnum = node->numportals; portalnum > 0; portalnum--)
            {
                tempportal = node->portallist.Get(portalnum);
                if (tempportal->PortalId == portal->PortalId)
                {
                    portal->frontleaf = originalnode;
                    portal->backleaf = node;
                    *flag += 1;
                }
            }
        }
        return;
    }
    else
    {
        CheckForSinglePortals(node->frontnode, originalnode, portal, flag);
        CheckForSinglePortals(node->backnode, originalnode, portal, flag);
        return;
    }
}

int RemoveExtraPortals(PORTAL* portal)
{
    int result, count = 0;
    POLYGON temppoly;
    BSP_node* tempnode = portal->backleaf;
    for (int loop = 0; loop < tempnode->numpolys; loop++)
    {
        temppoly = tempnode->nodepolylist[loop];
        result = ClassifyInvertedPortal(portal, temppoly);
        if (result == Front)
            count++;
    }
    if (count != portal->backleaf->numpolys)
        return true;
    else
        return false;
}

void FindTruePortals(BSP_node* node)
{
    int flag;
    if (node->leaf == true)
    {
        for (int portalnumber = node->numportals; portalnumber > 0; portalnumber--)
        {
            flag = 0;
            PORTAL* tempportal = node->portallist.Get(portalnumber);
            CheckForSinglePortals(root, node, tempportal, &flag);
            if (flag == 0)
            {
                delete[] tempportal->Vertex;
                node->portallist.Delete(portalnumber);
                node->numportals--;
            }
            else
            {
                ClipPortalToFrontLeaf(tempportal);
                ClipPortalToBackLeaf(tempportal);
            }
        }

        InvertPortals(node);    // also inverts the front and back leaf pointers if necessary

        for (int portalnumber = node->numportals; portalnumber > 0; portalnumber--)
        {
            PORTAL* tempportal = node->portallist.Get(portalnumber);
            flag = RemoveExtraPortals(tempportal);
            if (flag == true)
            {
                delete[] tempportal->Vertex;
                node->portallist.Delete(portalnumber);
                node->numportals--;
            }
        }
        return;
    }
    else
    {
        FindTruePortals(node->frontnode);
        FindTruePortals(node->backnode);
        return;
    }
}
